# 2DMAT -- Data-analysis software of quantum beam diffraction experiments for 2D material structure
# Copyright (C) 2020- The University of Tokyo
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see http://www.gnu.org/licenses/.

from typing import List

import collections
import numpy as np


def is_connected(nnlist: List[List[int]]) -> bool:
    nnodes = len(nnlist)
    visited = np.full(nnodes, False)
    nvisited = 1
    visited[0] = True
    stack = collections.deque([0])
    while len(stack) > 0:
        node = stack.pop()
        neighbors = [n for n in nnlist[node] if not visited[n]]
        visited[neighbors] = True
        stack.extend(neighbors)
        nvisited += len(neighbors)

    return nvisited == nnodes


def is_bidirectional(nnlist: List[List[int]]) -> bool:
    for i in range(len(nnlist)):
        for j in nnlist[i]:
            if i not in nnlist[j]:
                return False
    return True


if __name__ == "__main__":
    filename = "./neighborlist.txt"
    nnlist = []
    with open(filename) as f:
        for line in f:
            words = line.split()
            nn = [int(w) for w in words[1:]]
            nnlist.append(nn)
    print(is_connected(nnlist))
